/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};

var targetReady = false;

/**
 * This code implements the IPacketCodec interface.
 *
 * @constructor
 * @implements gc.databind.IPacketCodec
 */
CustomCodec = function()
{
};
CustomCodec.prototype.rxPktCtr = 0;
CustomCodec.prototype.rxMsgCtr = 0;
CustomCodec.prototype.txPktCtr = 0;
CustomCodec.prototype.txMsgCtr = 0;
CustomCodec.prototype.strRxBuffer = "";
CustomCodec.prototype.strTxBuffer = "";
CustomCodec.prototype.isConnect = false;
CustomCodec.prototype.initComplete = false;

/**
 * Encodes data into a packet for sending to the target device.
 *
 * @param target - function to call to pass on the encoded data towards the target
 * @param data - The data object from the GUI that is to be encoded into a packet for sending.
 */
CustomCodec.prototype.encode = function(target,data){
    this.txPktCtr++;

    /* The following example code shows how to decode JSON strings going down to the target
       into a Javascript object.  You would use this if you wanted to intercept data from the GUI
       and rework it before passing it on to the target device.
    */
    
     try {
            this.txMsgCtr++;
            if (this.txMsgCtr % 256){
                console.log("Number of tx messages to target: "+this.txMsgCtr);
            }

            // send the received and decoded javascript object to the application where you can use it to bind to widget properties
            var txStr = JSON.stringify(data) + '\n';
            target(txStr);

        }
        catch(ex){
            console.log("CustomCodec.encode (to target): strMessage="+strMessage+", exception="+ex);
        }
    
    
};
/**
 * Decodes packets from the target device into data objects.  One object for each packet of data.
 * Unless you have chained your custom codec with the CR codec, the packet data is not framed,
 * so this method is responsible for decoding partial packets as well as multiple packets.  Partial packets
 * should not be returned, but deferred until the next decode method call with more raw data to process.  As a
 * result, this method may return 0 or more packets of data on each invocation.
 *
 * @param target -  function to call to pass on the encoded data towards the GUI
 * @param data  - The raw data received that needs to be decoded from packets into objects.
 * @return - true if connected (i.e. valid data received), false if not connected
 */
CustomCodec.prototype.decode = function(target, data) {
    this.isConnected = true;
    this.rxPktCtr++;

    /* The following example code shows how to decode JSON strings coming up from the target
       into a Javascript object.  You would use this if you wanted to intercept data from the target
       and rework it before passing it on to the GUI for binding
*/
    var strRxData = data.map(function(value) { return String.fromCharCode(value);} ).join("");
    this.strRxBuffer = this.strRxBuffer+strRxData;
    var strMessage = '';
    while (this.strRxBuffer.indexOf('\n') !== -1) {
        strMessage = this.strRxBuffer.substring(0, this.strRxBuffer.indexOf('\n') + 1);
        this.strRxBuffer = this.strRxBuffer.substring(strMessage.length);

        try {
            var jsonObj = JSON.parse(strMessage);
            this.rxMsgCtr++;
            if (this.rxMsgCtr % 256){
                console.log("Number of tx messages from target: "+this.rxMsgCtr);
            }
            
            // Check if target device is ready to run demo
            if(jsonObj.Ready !== undefined)
            {
                targetReady = true;
                if(gc.services['ti-service-energytrace'])
                    templateObj.$.Play.disabled = false;
                stopTimerA();
            }
            
            // Check if target device is done running demo (CPU executes last)
            if(jsonObj.CPU_CYCLES !== undefined && jsonObj.CPU_CYCLES !== 0)
            {
                // Update cycle graph with CPU and LEA data
                templateObj.$.Cycles_Graph.series_0_values = jsonObj.CPU_CYCLES + ';' + " " + jsonObj.LEA_CYCLES +";";
                templateObj.$.CPU_LABEL.label = jsonObj.CPU_CYCLES;
                templateObj.$.LEA_LABEL.label = jsonObj.LEA_CYCLES;

                // Calculate the cycle and pwr improvement of LEA
                var cycleImprov = (jsonObj.CPU_CYCLES/jsonObj.LEA_CYCLES).toFixed(0);
                var pwrImprov = (aveCPUPwr/aveLEAPwr).toFixed(0);
                
                // Only display if improvement is positive
                if(cycleImprov > 1)
                {
                    templateObj.$.CYCLE_IMPROV.visible = true;
                    templateObj.$.CYCLE_IMPROV.label = cycleImprov + "X Improvement";
                }
                if(pwrImprov > 1)
                {
                    templateObj.$.PWR_IMPROV.visible = true;
                    templateObj.$.PWR_IMPROV.label = pwrImprov + "X Improvement"; 
                }
                
                // Demo is over, re-enable the play button and stop the timeout timer
                templateObj.$.Play.disabled = false;
                stopTimerA();
            }
            
            // Target device signaled that LEA is currently running a function
            if(jsonObj.LEA_RUNNING !== undefined)
            {
                // Start energy trace capture
                try {
                     if(jsonObj.LEA_RUNNING)
                     { // LEA just started
                        LEA_Running = true;
                        etCapture(4800);
                     }
                     else
                     { // LEA just ended
                         LEA_Running = false;
                         captureDone();
                     }
                }
                catch(ex){
                    console.log("Exception parsing string from target: ex="+ex);
                }
            }
            
            // Target device signaled that CPU is currently running a function
            if(jsonObj.CPU_RUNNING !== undefined)
            {
                try {
                     if(jsonObj.CPU_RUNNING)
                     { // CPU just started
                        LEA_Running = false;
                        etCapture(4800);
                     }
                     else
                     { // CPU just ended
                         LEA_Running = false;
                         captureDone();
                     }
                }
                catch(ex){
                    console.log("Exception parsing string from target: ex="+ex);
                }
            }
            
            // send the received and decoded javascript object to the application where you can use it to bind to widget properties
            target(jsonObj);
        }
        catch(ex){
            console.log("CustomCodec.decode (from target): strMessage="+strMessage+", exception="+ex);
        }
    }
    
    // Return true to set the 'Hardware Connected' status in the status bar
    return this.isConnected;

};

var close = function(){
    gc.databind.unregisterCustomCodec("custom",CustomCodec);
    CustomCodec.initComplete = false;
    CustomCodec.isConnected = false;
};

// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {
    // You can 'chain' codecs together to make it easier to work with the data
    // coming from the device or going to the device.
    // The following codecs are provided:
    //   CR  - use this if the target data uses /n as a delimiter for framing.
    //         The CR codec will take care of buffering the incoming data so that
    //         your codec will only receive complete 'ready to use' packets
    //  If your target uses /n delimited strings, set baseCodecs = "CR"
    //  If your target uses some other format for data (e.g. binary data or custom strings) leave baseCodecs = "undefined"

    var baseCodecs = undefined;
    console.log("custom codec: about to register..."); // log a message that you will be able to see in the debugger
    gc.databind.registerCustomCodec("custom",CustomCodec, baseCodecs);
    CustomCodec.initComplete = true;
};

templateObj = document.querySelector('#template_obj');
if (templateObj) {
    init();
} else {
    document.addEventListener('DOMContentLoaded',init.bind(this))
}
